import { CommonModule } from '@angular/common';
import { Component, HostBinding, Input, InjectionToken, Directive, ElementRef, Host, Optional, SkipSelf, ContentChild, Inject, EventEmitter, HostListener, Output, NgModule } from '@angular/core';
import { ControlContainer, FormControl, FormArray, FormGroup, NgForm } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValidationFeedbackComponent {
    constructor() {
        this.messages = () => [];
    }
    // TODO support valid-feedback as well
    /**
     * @return {?}
     */
    get classes() {
        return true;
    }
}
ValidationFeedbackComponent.decorators = [
    { type: Component, args: [{
                selector: 'wp-validation-feedback',
                template: "<span *ngFor=\"let message of messages()\">{{message}}</span>",
                styles: [".invalid-feedback,.valid-feedback{display:block}"]
            }] }
];
/** @nocollapse */
ValidationFeedbackComponent.ctorParameters = () => [];
ValidationFeedbackComponent.propDecorators = {
    messages: [{ type: Input }],
    classes: [{ type: HostBinding, args: ['class.invalid-feedback',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_ERRORS = [
    {
        error: 'required',
        format: label => `${label} is required.`
    },
    {
        error: 'pattern',
        format: label => `${label} is invalid.`
    },
    {
        error: 'minlength',
        format: (label, error) => `${label} must be at least ${error.requiredLength} characters`
    },
    {
        error: 'maxlength',
        format: (label, error) => `${label} must be no longer than ${error.requiredLength} characters`
    },
    {
        error: 'requiredTrue',
        format: (label) => `${label} is required`
    },
    {
        error: 'email',
        format: () => `Invalid email address`
    },
    {
        error: 'max',
        format: (label, error) => `${label} must be no greater than ${error.max}`
    },
    {
        error: 'min',
        format: (label, error) => `${label} must be no less than ${error.min}`
    }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const CUSTOM_ERROR_MESSAGES = new InjectionToken('ng-bootstrap-validation-form-error-messages');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} name
 * @param {?} parent
 * @return {?}
 */
function controlPath(name, parent) {
    // tslint:disable-next-line:no-non-null-assertion
    return [...(/** @type {?} */ (parent.path)), name];
}
class FormControlDirective {
    /**
     * @param {?} parent
     * @param {?} formControl
     * @param {?} elementRef
     */
    constructor(parent, formControl, elementRef) {
        this.parent = parent;
        this.formControl = formControl;
        this.elementRef = elementRef;
    }
    /**
     * @return {?}
     */
    get name() {
        if (this.formControlName) {
            return this.formControlName;
        }
        this.formControlName = this.elementRef.nativeElement.getAttribute('name');
        return this.formControlName;
    }
    // This method will be invoked by "getControl" method on "formDirective"
    /**
     * @return {?}
     */
    get path() {
        if (!this.formControlName) {
            this.formControlName = this.elementRef.nativeElement.getAttribute('name');
        }
        return controlPath(this.name, this.parent);
    }
    /**
     * @return {?}
     */
    get validClass() {
        if (!this.control) {
            return false;
        }
        return this.control.valid && (this.control.touched || this.control.dirty);
    }
    /**
     * @return {?}
     */
    get invalidClass() {
        if (!this.control) {
            return false;
        }
        return this.control.invalid && this.control.touched && this.control.dirty;
    }
    /**
     * @return {?}
     */
    get control() {
        return this.formDirective && this.formDirective.getControl(this);
    }
    /**
     * @return {?}
     */
    get formDirective() {
        return this.parent ? this.parent.formDirective : null;
    }
}
FormControlDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '.form-control,.form-check-input,.custom-control-input,[wpFormControl]'
            },] }
];
/** @nocollapse */
FormControlDirective.ctorParameters = () => [
    { type: ControlContainer, decorators: [{ type: Optional }, { type: Host }, { type: SkipSelf }] },
    { type: FormControl, decorators: [{ type: Optional }, { type: Host }] },
    { type: ElementRef }
];
FormControlDirective.propDecorators = {
    formControlName: [{ type: Input }],
    validClass: [{ type: HostBinding, args: ['class.is-valid',] }],
    invalidClass: [{ type: HostBinding, args: ['class.is-invalid',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormGroupComponent {
    /**
     * @param {?} elRef
     * @param {?} customErrorMessages
     */
    constructor(elRef, customErrorMessages) {
        this.elRef = elRef;
        this.customErrorMessages = customErrorMessages;
        this.messages = () => this.getMessages();
    }
    /**
     * @return {?}
     */
    get hasErrors() {
        if (!this.formControlDirective) {
            return false;
        }
        return this.formControlDirective.control &&
            !this.formControlDirective.control.valid &&
            this.formControlDirective.control.dirty &&
            this.formControlDirective.control.touched;
    }
    /**
     * @return {?}
     */
    get hasSuccess() {
        if (!this.formControlDirective) {
            return false;
        }
        return this.formControlDirective.control &&
            this.formControlDirective.control.valid &&
            this.formControlDirective.control.dirty &&
            this.formControlDirective.control.touched;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this.messagesBlock) {
            this.messagesBlock.messages = this.messages;
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.errorMessages = DEFAULT_ERRORS
            .concat(this.customErrorMessages)
            .reverse();
    }
    /**
     * @return {?}
     */
    get label() {
        /** @type {?} */
        const label = this.elRef.nativeElement.querySelector('label');
        return label && label.textContent ? label.textContent.trim() : 'This field';
    }
    /**
     * @return {?}
     */
    get isDirtyAndTouched() {
        if (!this.formControlDirective) {
            return false;
        }
        return this.formControlDirective.control &&
            this.formControlDirective.control.dirty &&
            this.formControlDirective.control.touched;
    }
    /**
     * @private
     * @return {?}
     */
    getMessages() {
        /** @type {?} */
        const messages = [];
        if (!this.isDirtyAndTouched) {
            return messages;
        }
        /** @type {?} */
        const errors = this.formControlDirective.control.errors;
        if (!errors) {
            return;
        }
        Object.keys(this.formControlDirective.control.errors).forEach(key => {
            /** @type {?} */
            const error = this.errorMessages.find(err => err.error === key);
            if (!error) {
                return;
            }
            messages.push(error.format(this.label, this.formControlDirective.control.errors[key]));
        });
        return messages;
    }
}
FormGroupComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable:component-selector
                selector: '.form-group,[wpFormGroup]',
                template: "<ng-content></ng-content>\n<wp-validation-feedback *ngIf=\"!messagesBlock\" [messages]=\"messages\"></wp-validation-feedback>"
            }] }
];
/** @nocollapse */
FormGroupComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Array, decorators: [{ type: Inject, args: [CUSTOM_ERROR_MESSAGES,] }] }
];
FormGroupComponent.propDecorators = {
    formControlDirective: [{ type: ContentChild, args: [FormControlDirective,] }],
    messagesBlock: [{ type: ContentChild, args: [ValidationFeedbackComponent,] }],
    hasErrors: [{ type: HostBinding, args: ['class.has-error',] }],
    hasSuccess: [{ type: HostBinding, args: ['class.has-success',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormDirective {
    /**
     * @param {?} ngForm
     * @param {?} formGroup
     */
    constructor(ngForm, formGroup) {
        this.ngForm = ngForm;
        this.formGroup = formGroup;
        // https://stackoverflow.com/a/40706065/2405040
        this.validFormSubmit = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.formGroup && this.ngForm) {
            this.formGroup = this.ngForm.form;
        }
    }
    /**
     * @return {?}
     */
    onSubmit() {
        if (!this.formGroup) {
            console.warn('Could not find any formGroup or ngForm on the');
            return;
        }
        this.markAsTouchedAndDirty(this.formGroup);
        if (this.formGroup.valid) {
            this.validFormSubmit.emit(this.formGroup.value);
        }
    }
    /**
     * @private
     * @param {?} control
     * @return {?}
     */
    markAsTouchedAndDirty(control) {
        if (control instanceof FormGroup) {
            Object.keys(control.controls).forEach((key) => {
                this.markAsTouchedAndDirty(control.controls[key]);
            });
        }
        else if (control instanceof FormArray) {
            control.controls.forEach(c => this.markAsTouchedAndDirty(c));
        }
        else if (control instanceof FormControl && control.enabled) {
            control.markAsDirty();
            control.markAsTouched();
            control.updateValueAndValidity();
        }
    }
}
FormDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wpForm]'
            },] }
];
/** @nocollapse */
FormDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }, { type: Host }] },
    { type: FormGroup, decorators: [{ type: Optional }, { type: Host }] }
];
FormDirective.propDecorators = {
    wpForm: [{ type: Input }],
    validFormSubmit: [{ type: Output }],
    onSubmit: [{ type: HostListener, args: ['submit',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = (form) => {
    return form;
};
// @dynamic
class ParentFormDirective {
}
ParentFormDirective.decorators = [
    { type: Directive, args: [{
                selector: '[wpLinkParentForm]',
                providers: [
                    {
                        provide: ControlContainer,
                        useFactory: ɵ0,
                        deps: [NgForm]
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapValidationModule {
    /**
     * @param {?} customErrorMessages
     * @return {?}
     */
    static forRoot(customErrorMessages) {
        return {
            ngModule: NgxBootstrapValidationModule,
            providers: [
                {
                    provide: CUSTOM_ERROR_MESSAGES,
                    useValue: customErrorMessages || [],
                    multi: true
                }
            ]
        };
    }
}
NgxBootstrapValidationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    FormControlDirective,
                    FormDirective,
                    ParentFormDirective,
                    ValidationFeedbackComponent,
                    FormGroupComponent
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    FormControlDirective,
                    FormDirective,
                    ParentFormDirective,
                    ValidationFeedbackComponent,
                    FormGroupComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FormGroupComponent, ValidationFeedbackComponent, controlPath, FormControlDirective, FormDirective, NgxBootstrapValidationModule, ParentFormDirective as ɵc, CUSTOM_ERROR_MESSAGES as ɵa };

//# sourceMappingURL=wizpanda-ngx-bootstrap-validation.js.map